// ################################################################################
// Common Framework for Computer Graphics Courses at FI MUNI.
// 
// Copyright (c) 2021-2022 Visitlab (https://visitlab.fi.muni.cz)
// All rights reserved.
// ################################################################################

#pragma once

#include "cube.hpp"
#include "pv112_application.hpp"
#include "sphere.hpp"
#include "teapot.hpp"

class Application : public PV112Application {

    // ----------------------------------------------------------------------------
    // Variables
    // ----------------------------------------------------------------------------
private:
    GLuint main_program = 0;

    Cube cube{};
    Sphere sphere{};
    Teapot teapot{};

    glm::vec3 cam_position = glm::vec3(1.5f, 2.0f, -8.0f);
    glm::vec3 light_position = glm::vec3(0.f, 3.0f, 0.f);
    float light = 1.0f; //1 = spot light; 0 = directional

    glm::vec3 light_ambient_color  = glm::vec3(1.f, 1.f, 1.f);
    glm::vec3 light_diffuse_color  = glm::vec3(1.f, 1.f, 1.f);
    glm::vec3 light_specular_color = glm::vec3(1.f, 1.f, 1.f);

    glm::vec3 object_ambient_color  = glm::vec3(1.f, 0.f, 0.f);
    glm::vec3 object_diffuse_color  = glm::vec3(0.f, 1.f, 0.f);
    glm::vec3 object_specular_color = glm::vec3(0.f, 0.f, 1.f);

    // ----------------------------------------------------------------------------
    // Constructors & Destructors
    // ----------------------------------------------------------------------------
public:
    /**
     * Constructs a new @link Application with a custom width and height.
     *
     * @param 	initial_width 	The initial width of the window.
     * @param 	initial_height	The initial height of the window.
     * @param 	arguments	  	The command line arguments used to obtain the application directory.
     */
    Application(int initial_width, int initial_height, std::vector<std::string> arguments = {});

    /** Destroys the {@link Application} and releases the allocated resources. */
    virtual ~Application();

    // ----------------------------------------------------------------------------
    // Methods
    // ----------------------------------------------------------------------------

    /** @copydoc PV112Application::compile_shaders */
    void compile_shaders() override;

    /** @copydoc PV112Application::delete_shaders */
    void delete_shaders() override;

    /** @copydoc PV112Application::update */
    void update(float delta) override;

    /** @copydoc PV112Application::render */
    void render() override;

    /** @copydoc PV112Application::render_ui */
    void render_ui() override;

    // ----------------------------------------------------------------------------
    // Input Events
    // ----------------------------------------------------------------------------

    /** @copydoc PV112Application::on_resize */
    void on_resize(int width, int height) override;

    /** @copydoc PV112Application::on_mouse_move */
    void on_mouse_move(double x, double y) override;

    /** @copydoc PV112Application::on_mouse_button */
    void on_mouse_button(int button, int action, int mods) override;

    /** @copydoc PV112Application::on_key_pressed */
    void on_key_pressed(int key, int scancode, int action, int mods) override;
};
