#version 450

// ----------------------------------------------------------------------------
// Input Variables
// ----------------------------------------------------------------------------

// Task 5.2: Add a uniform variable for eye(camera) position.
layout(location = 3) uniform vec3 cam_position;
// Task 5.2: Add a uniform variable for light's position and ambient+diffuse+specular terms.
layout(location = 4) uniform vec4 light_position;

// Task 5.2: Add a uniform variable for material's ambient+diffuse+specular+shininess terms.
layout(location = 5) uniform vec3 light_ambient_color;
layout(location = 6) uniform vec3 light_diffuse_color;
layout(location = 7) uniform vec3 light_specular_color;

layout(location = 8) uniform vec3 object_ambient_color;
layout(location = 9) uniform vec3 object_diffuse_color;
layout(location = 10) uniform vec3 object_specular_color;

// The intpolated position from the vertex shader.
layout(location = 0) in vec3 fs_position;
// The intpolated normal from the vertex shader.
layout(location = 1) in vec3 fs_normal;
// The intpolated texture coordinates from the vertex shader.
layout(location = 2) in vec2 fs_texture_coordinate;

// ----------------------------------------------------------------------------
// Output Variables
// ----------------------------------------------------------------------------

// The final output color.
layout(location = 0) out vec4 final_color;

// ----------------------------------------------------------------------------
// Main Method
// ----------------------------------------------------------------------------
void main()
{
	// Task 5.2: Compute (N)ormal, (L)ight, V = Eye, (H)alf vectors.
	vec3 N = normalize(fs_normal);
	vec3 L = normalize(light_position.xyz - fs_position);
	vec3 V = normalize(cam_position - fs_position);
	vec3 H = normalize(V + L);

	// Task 5.2: Compute dot products between N and L, N and H.
	float NdotL = max( dot(N,L), 0.0 );
	float NdotH = max( dot(N,H), 0.0 );

	// Task 5.2: Compute material values multiplied by light's values.
	// Task 5.2: Compute final color using Blinn�Phong shading model.
	// Task 5.2: Use attenuation based on the light distance for point lights.
	// Task 5.2: Differentiate between directional and point light based on its .w value in position.

	//this gives the final mixed color per all three elements of a color
	vec3 final_ambient_color = light_ambient_color * object_ambient_color;
	vec3 final_diffuse_color = light_diffuse_color * object_diffuse_color;
	vec3 final_specular_color = light_specular_color * object_specular_color;

	vec3 tmp_color = final_ambient_color + NdotL*final_diffuse_color + pow(NdotH, 10.0)*final_specular_color;

	float distance2 = light_position.w == 1.0 ? pow(length(light_position.xyz - fs_position),2) : 1.0;
	final_color = vec4(1.0/distance2 * tmp_color, 1.0);
}
